# encoding: utf-8
import json

from items import ITEMS, check


KEY_TREATLIST = 'treatlist'
KEY_EXPECT_DATE = 'expectdate'
KEY_CHECKS = 'form'


class Treat:
    """Represents a patient's one treatment record."""

    def __init__(self, json_obj, patient_name, fix):
        self.data = json_obj
        self.patient_name = patient_name
        self.expected = self.data[KEY_EXPECT_DATE]
        self.fix = fix

    def check(self):
        valid = True
        for item in self.data[KEY_CHECKS]:
            if item['item'] in ITEMS.keys():
                if not check(item, self.patient_name, self.expected, self.fix):
                    valid = False

        if self.fix and not valid:
            self.update()

        return valid

    def update(self):
        """update treatment status from `complete` to `im-progress`"""
        self.data['classType'] = 'in-progress'
        self.data['fix-info'] = 'fix by NDIP checker.'


class Treatment:
    """Represents a patient's all treatment record."""

    def __init__(self, record_id, patient_id, patient_name, json_obj, fix=False):
        """init treatment."""
        self.record_id = record_id
        self.patient_id = patient_id
        self.patient_name = patient_name
        self.data = json_obj
        self.fix = fix

    def check(self, db):
        valid = True
        for data in self.data[KEY_TREATLIST]:
            treat = Treat(data, self.patient_name, self.fix)
            if not treat.check():
                valid = False

        if self.fix and not valid:
            self.update(db)

        return valid

    def update(self, db):
        """Update the treatment record to database."""
        data = json.dumps(self.data)
        sql = "UPDATE hospital_record set treatment=%s where id=%s"
        val = (data, self.record_id)

        cur = db.cursor()
        cur.execute(sql, val)

        db.commit()
        print(cur.rowcount, " record(s) affected")



# encoding: utf-8

from . import BingChengJiLuItem, TuPianItem, PeiYangItem


def generate_peiyang(*args):
    return PeiYangItem(*args)


def generate_tupian(*args):
    return TuPianItem(*args)


def generate_bjcl(*args):
    return BingChengJiLuItem(*args)


ITEMS = {u'培养': generate_peiyang, u'涂片': generate_tupian, u'病程记录': generate_bjcl}


def check(item, patient_name, expected_date, fix=False):
    valid = True
    if item['item'] in ITEMS.keys() and 'data' in item:
        obj = ITEMS[item['item']](item, patient_name, expected_date)
        if not obj.is_valid():
            valid = False
            if fix:
                obj.update()

    return valid



# encoding: utf-8
import argparse


def get_parser():
    """get command parser."""
    parser = argparse.ArgumentParser(
        description="check and fix errors in NDIP",
        epilog="""
        check/fix 涂片/培养/病程记录
        """
    )
    parser.add_argument("-o", "--host",
                        default="localhost",
                        help="mysql host, default localhost")
    parser.add_argument("-u", "--user",
                        default="root",
                        help="mysql user, default root")
    parser.add_argument("-p", "--password",
                        default="ChangeMe1!",
                        help="mysql password, default ChangeMe1!")
    parser.add_argument("-d", "--database",
                        default="hospitalAdmin",
                        help="mysql database, default hospitalAdmin")
    parser.add_argument("-f", "--fix",
                        action="store_true",
                        help="whether fix error, default False.")
    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    print(args.host)
    print(args.user)
    print(args.password)
    print(args.database)
    print(args.fix)





# encoding: utf-8
import json
import MySQLdb

from parser import get_parser
from treatment import Treatment


def execute_from_command_line():
    """run script"""
    parser = get_parser()
    args = parser.parse_args()

    host = args.host
    user = args.user
    password = args.password
    db_name = args.database
    fix = args.fix
    db = MySQLdb.connect(host=host, user=user, passwd=password, db=db_name, charset='utf8')

    cur = db.cursor()
    cur.execute("select id, treatment, patient_id from hospital_record")

    for row in cur.fetchall():
        record_id = row[0]
        data = json.loads(row[1])
        patient_id = row[2]
        cur2 = db.cursor()
        sql = "select code from hospital_patient where id={}".format(patient_id)
        cur2.execute(sql)
        patient_name = cur2.fetchall()[0][0]

        treatment = Treatment(record_id, patient_id, patient_name, data, fix)
        treatment.check(db)

    db.close()


if __name__ == "__main__":
    execute_from_command_line()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ndip_checker",
    version="1.0.0",
    author="Yarving Liu",
    author_email="yarving@gmail.com",
    description="check and fix errors in NDIP",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/yarving/NdipChecker",
    packages=setuptools.find_packages(),
    scripts=['ndip_checker/main.py'],
    entry_points={'console_scripts': [
            'ncheck=ndip_checker:execute_from_command_line',
    ]},
    install_requires=['MySQL-python'],
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# encoding: utf-8
import uuid

from . import BaseItem, KEY_FIX_INFO, KEY_ID, KEY_FINISH_TAG, KEY_CLASS_STATUS


class BingChengJiLuItem(BaseItem):
    """培养检查项 """

    def __init__(self, json_obj, patient_name, expected_date):
        self.json_obj = json_obj
        self.data = json_obj['data']
        self.patient_name = patient_name
        self.expected_date = expected_date

    def is_valid(self):
        if isinstance(self.data, dict):
            name = u"病程记录"
            msg = u"{} 预计 {} 写的 {} 格式不对: {}".format(
                self.patient_name, self.expected_date, name, self.data )
            print(msg)
            return False

        return True

    def update(self):
        """make data correct and clean."""

        # fix and clean data
        self.json_obj['data'] = ''

        self.json_obj[KEY_FIX_INFO] = 'fix by NDIP checker'

        # add identification if null
        if KEY_ID not in self.json_obj:
            self.json_obj[KEY_ID] = str(uuid.uuid4()).replace('-', '')

        # fix status
        self.json_obj[KEY_FINISH_TAG] = False
        self.json_obj[KEY_CLASS_STATUS] = 'not-start'



KEY_ID = 'identification'
KEY_FIX_INFO = 'fix_info'
KEY_FINISH_TAG = 'is_finished'
KEY_CLASS_STATUS = 'class'


from .base import BaseItem
from .bingchengjilu import BingChengJiLuItem
from .tupian import TuPianItem
from .peiyang import PeiYangItem
from .factory import ITEMS, check


# encoding: utf-8
import uuid

from . import BaseItem, KEY_FIX_INFO, KEY_ID, KEY_FINISH_TAG, KEY_CLASS_STATUS


class PeiYangItem(BaseItem):
    """培养检查项 """

    items = ['date1', 'date2', 'pyResult', 'method', 'origin', 'pyChecked']

    def __init__(self, json_obj, patient_name, expected_date):
        self.json_obj = json_obj
        self.data = json_obj['data']
        self.patient_name = patient_name
        self.expected_date = expected_date

    def is_valid(self):
        name = u"培养"
        if not isinstance(self.data, dict):
            msg = u"{} 预计 {} 写的 {} 不是字典格式: {}".format(
                self.patient_name, self.expected_date, name, self.data )
            print(msg)
            return False

        for item in PeiYangItem.items:
            try:
                self.data[item]
            except KeyError:
                msg = u"{} 预计 {} 写的 {} 格式不对: {}".format(
                    self.patient_name, self.expected_date, name, self.data )
                print(msg)
                return False

        if not set(PeiYangItem.items).issubset(set(self.data.keys())):
            msg = u"{} 预计 {} 写的 {} 键值不对: {}".format(
                self.patient_name, self.expected_date, name, self.data )
            print(msg)
            return False

        return True

    def update(self):
        """correct and clean data."""

        # fix and clean data
        self.json_obj['data'] = {}
        for item in PeiYangItem.items:
            self.json_obj['data'][item] = ''

        self.json_obj[KEY_FIX_INFO] = 'fix by NDIP checker'

        # add identification if null
        if KEY_ID not in self.json_obj:
            self.json_obj[KEY_ID] = str(uuid.uuid4()).replace('-', '')

        # fix status
        self.json_obj[KEY_FINISH_TAG] = False
        self.json_obj[KEY_CLASS_STATUS] = 'not-start'
        self.data = {}
        for item in PeiYangItem.items:
            self.data[item] = ''
